@@ -1,21 +1,18 @@
 """
 Testing for the nearest centroid module.
 """
-
 import numpy as np
-from scipy import sparse as sp
+import pytest
 from numpy.testing import assert_array_equal
 
-from sklearn.neighbors import NearestCentroid
 from sklearn import datasets
-from sklearn.utils._testing import assert_raises
+from sklearn.neighbors import NearestCentroid
+from sklearn.utils.fixes import CSR_CONTAINERS
 
 # toy sample
 X = [[-2, -1], [-1, -1], [-1, -2], [1, 1], [1, 2], [2, 1]]
-X_csr = sp.csr_matrix(X)  # Sparse matrix
 y = [-1, -1, -1, 1, 1, 1]
 T = [[-1, -1], [2, 2], [3, 2]]
-T_csr = sp.csr_matrix(T)
 true_result = [-1, 1, 1]
 
 # also load the iris dataset
@@ -27,8 +24,12 @@ iris.data = iris.data[perm]
 iris.target = iris.target[perm]
 
 
-def test_classification_toy():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_classification_toy(csr_container):
     # Check classification on a toy dataset, including sparse versions.
+    X_csr = csr_container(X)
+    T_csr = csr_container(T)
+
     clf = NearestCentroid()
     clf.fit(X, y)
     assert_array_equal(clf.predict(T), true_result)
@@ -54,26 +55,23 @@ def test_classification_toy():
     assert_array_equal(clf.predict(T_csr.tolil()), true_result)
 
 
-def test_precomputed():
-    clf = NearestCentroid(metric='precomputed')
-    with assert_raises(ValueError):
-        clf.fit(X, y)
-
-
+# TODO(1.5): Remove filterwarnings when support for some metrics is removed
+@pytest.mark.filterwarnings("ignore:Support for distance metrics:FutureWarning:sklearn")
 def test_iris():
     # Check consistency on dataset iris.
-    for metric in ('euclidean', 'cosine'):
+    for metric in ("euclidean", "cosine"):
         clf = NearestCentroid(metric=metric).fit(iris.data, iris.target)
         score = np.mean(clf.predict(iris.data) == iris.target)
         assert score > 0.9, "Failed with score = " + str(score)
 
 
+# TODO(1.5): Remove filterwarnings when support for some metrics is removed
+@pytest.mark.filterwarnings("ignore:Support for distance metrics:FutureWarning:sklearn")
 def test_iris_shrinkage():
     # Check consistency on dataset iris, when using shrinkage.
-    for metric in ('euclidean', 'cosine'):
+    for metric in ("euclidean", "cosine"):
         for shrink_threshold in [None, 0.1, 0.5]:
-            clf = NearestCentroid(metric=metric,
-                                  shrink_threshold=shrink_threshold)
+            clf = NearestCentroid(metric=metric, shrink_threshold=shrink_threshold)
             clf = clf.fit(iris.data, iris.target)
             score = np.mean(clf.predict(iris.data) == iris.target)
             assert score > 0.8, "Failed with score = " + str(score)
@@ -91,9 +89,11 @@ def test_pickle():
     obj2 = pickle.loads(s)
     assert type(obj2) == obj.__class__
     score2 = obj2.score(iris.data, iris.target)
-    assert_array_equal(score, score2,
-                       "Failed to generate same score"
-                       " after pickling (classification).")
+    assert_array_equal(
+        score,
+        score2,
+        "Failed to generate same score after pickling (classification).",
+    )
 
 
 def test_shrinkage_correct():
@@ -137,12 +137,42 @@ def test_predict_translated_data():
     assert_array_equal(y_init, y_translate)
 
 
-def test_manhattan_metric():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_manhattan_metric(csr_container):
     # Test the manhattan metric.
+    X_csr = csr_container(X)
 
-    clf = NearestCentroid(metric='manhattan')
+    clf = NearestCentroid(metric="manhattan")
     clf.fit(X, y)
     dense_centroid = clf.centroids_
     clf.fit(X_csr, y)
     assert_array_equal(clf.centroids_, dense_centroid)
     assert_array_equal(dense_centroid, [[-1, -1], [1, 1]])
+
+
+# TODO(1.5): remove this test
+@pytest.mark.parametrize(
+    "metric", sorted(list(NearestCentroid._valid_metrics - {"manhattan", "euclidean"}))
+)
+def test_deprecated_distance_metric_supports(metric):
+    # Check that a warning is raised for all deprecated distance metric supports
+    clf = NearestCentroid(metric=metric)
+    with pytest.warns(
+        FutureWarning,
+        match="Support for distance metrics other than euclidean and manhattan",
+    ):
+        clf.fit(X, y)
+
+
+def test_features_zero_var():
+    # Test that features with 0 variance throw error
+
+    X = np.empty((10, 2))
+    X[:, 0] = -0.13725701
+    X[:, 1] = -0.9853293
+    y = np.zeros((10))
+    y[0] = 1
+
+    clf = NearestCentroid(shrink_threshold=0.1)
+    with pytest.raises(ValueError):
+        clf.fit(X, y)
