@@ -37,35 +37,27 @@ from sklearn.utils.extmath import (
     svd_flip,
     weighted_mode,
 )
-from sklearn.utils.fixes import _mode
+from sklearn.utils.fixes import (
+    COO_CONTAINERS,
+    CSC_CONTAINERS,
+    CSR_CONTAINERS,
+    DOK_CONTAINERS,
+    LIL_CONTAINERS,
+    _mode,
+)
 
 
-def test_density():
+@pytest.mark.parametrize(
+    "sparse_container",
+    COO_CONTAINERS + CSC_CONTAINERS + CSR_CONTAINERS + LIL_CONTAINERS,
+)
+def test_density(sparse_container):
     rng = np.random.RandomState(0)
     X = rng.randint(10, size=(10, 5))
     X[1, 2] = 0
     X[5, 3] = 0
-    X_csr = sparse.csr_matrix(X)
-    X_csc = sparse.csc_matrix(X)
-    X_coo = sparse.coo_matrix(X)
-    X_lil = sparse.lil_matrix(X)
 
-    for X_ in (X_csr, X_csc, X_coo, X_lil):
-        assert density(X_) == density(X)
-
-
-# TODO(1.4): Remove test
-def test_density_deprecated_kwargs():
-    """Check that future warning is raised when user enters keyword arguments."""
-    test_array = np.array([[1, 2, 3], [4, 5, 6]])
-    with pytest.warns(
-        FutureWarning,
-        match=(
-            "Additional keyword arguments are deprecated in version 1.2 and will be"
-            " removed in version 1.4."
-        ),
-    ):
-        density(test_array, a=1)
+    assert density(sparse_container(X)) == density(X)
 
 
 def test_uniform_weights():
@@ -100,7 +92,8 @@ def test_random_weights():
     assert_array_almost_equal(score.ravel(), w[:, :5].sum(1))
 
 
-def check_randomized_svd_low_rank(dtype):
+@pytest.mark.parametrize("dtype", (np.int32, np.int64, np.float32, np.float64))
+def test_randomized_svd_low_rank_all_dtypes(dtype):
     # Check that extmath.randomized_svd is consistent with linalg.svd
     n_samples = 100
     n_features = 500
@@ -160,27 +153,23 @@ def check_randomized_svd_low_rank(dtype):
         )
 
         # check the sparse matrix representation
-        X = sparse.csr_matrix(X)
+        for csr_container in CSR_CONTAINERS:
+            X = csr_container(X)
 
-        # compute the singular values of X using the fast approximate method
-        Ua, sa, Va = randomized_svd(
-            X, k, power_iteration_normalizer=normalizer, random_state=0
-        )
-        if dtype.kind == "f":
-            assert Ua.dtype == dtype
-            assert sa.dtype == dtype
-            assert Va.dtype == dtype
-        else:
-            assert Ua.dtype.kind == "f"
-            assert sa.dtype.kind == "f"
-            assert Va.dtype.kind == "f"
+            # compute the singular values of X using the fast approximate method
+            Ua, sa, Va = randomized_svd(
+                X, k, power_iteration_normalizer=normalizer, random_state=0
+            )
+            if dtype.kind == "f":
+                assert Ua.dtype == dtype
+                assert sa.dtype == dtype
+                assert Va.dtype == dtype
+            else:
+                assert Ua.dtype.kind == "f"
+                assert sa.dtype.kind == "f"
+                assert Va.dtype.kind == "f"
 
-        assert_almost_equal(s[:rank], sa[:rank], decimal=decimal)
-
-
-@pytest.mark.parametrize("dtype", (np.int32, np.int64, np.float32, np.float64))
-def test_randomized_svd_low_rank_all_dtypes(dtype):
-    check_randomized_svd_low_rank(dtype)
+            assert_almost_equal(s[:rank], sa[:rank], decimal=decimal)
 
 
 @pytest.mark.parametrize("dtype", (np.int32, np.int64, np.float32, np.float64))
@@ -316,7 +305,8 @@ def test_randomized_eigsh_reconst_low_rank(n, rank):
 
 
 @pytest.mark.parametrize("dtype", (np.float32, np.float64))
-def test_row_norms(dtype):
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_row_norms(dtype, csr_container):
     X = np.random.RandomState(42).randn(100, 100)
     if dtype is np.float32:
         precision = 4
@@ -330,7 +320,7 @@ def test_row_norms(dtype):
     assert_array_almost_equal(np.sqrt(sq_norm), row_norms(X), precision)
 
     for csr_index_dtype in [np.int32, np.int64]:
-        Xcsr = sparse.csr_matrix(X, dtype=dtype)
+        Xcsr = csr_container(X, dtype=dtype)
         # csr_matrix will use int32 indices by default,
         # up-casting those to int64 when necessary
         if csr_index_dtype is np.int64:
@@ -501,19 +491,21 @@ def test_randomized_svd_power_iteration_normalizer():
             assert 15 > np.abs(error_2 - error)
 
 
-def test_randomized_svd_sparse_warnings():
+@pytest.mark.parametrize("sparse_container", DOK_CONTAINERS + LIL_CONTAINERS)
+def test_randomized_svd_sparse_warnings(sparse_container):
     # randomized_svd throws a warning for lil and dok matrix
     rng = np.random.RandomState(42)
     X = make_low_rank_matrix(50, 20, effective_rank=10, random_state=rng)
     n_components = 5
-    for cls in (sparse.lil_matrix, sparse.dok_matrix):
-        X = cls(X)
-        warn_msg = (
-            "Calculating SVD of a {} is expensive. "
-            "csr_matrix is more efficient.".format(cls.__name__)
+
+    X = sparse_container(X)
+    warn_msg = (
+        "Calculating SVD of a {} is expensive. csr_matrix is more efficient.".format(
+            sparse_container.__name__
         )
-        with pytest.warns(sparse.SparseEfficiencyWarning, match=warn_msg):
-            randomized_svd(X, n_components, n_iter=1, power_iteration_normalizer="none")
+    )
+    with pytest.warns(sparse.SparseEfficiencyWarning, match=warn_msg):
+        randomized_svd(X, n_components, n_iter=1, power_iteration_normalizer="none")
 
 
 def test_svd_flip():
@@ -541,6 +533,21 @@ def test_svd_flip():
     assert_almost_equal(np.dot(U_flip2 * S, V_flip2), XT, decimal=6)
 
 
+@pytest.mark.parametrize("n_samples, n_features", [(3, 4), (4, 3)])
+def test_svd_flip_max_abs_cols(n_samples, n_features, global_random_seed):
+    rs = np.random.RandomState(global_random_seed)
+    X = rs.randn(n_samples, n_features)
+    U, _, Vt = linalg.svd(X, full_matrices=False)
+
+    U1, _ = svd_flip(U, Vt, u_based_decision=True)
+    max_abs_U1_row_idx_for_col = np.argmax(np.abs(U1), axis=0)
+    assert (U1[max_abs_U1_row_idx_for_col, np.arange(U1.shape[1])] >= 0).all()
+
+    _, V2 = svd_flip(U, Vt, u_based_decision=False)
+    max_abs_V2_col_idx_for_row = np.argmax(np.abs(V2), axis=1)
+    assert (V2[np.arange(V2.shape[0]), max_abs_V2_col_idx_for_row] >= 0).all()
+
+
 def test_randomized_svd_sign_flip():
     a = np.array([[2.0, 0.0], [0.0, 1.0]])
     u1, s1, v1 = randomized_svd(a, 2, flip_sign=True, random_state=41)
@@ -666,16 +673,20 @@ def test_cartesian_mix_types(arrays, output_dtype):
     assert output.dtype == output_dtype
 
 
+# TODO(1.6): remove this test
 def test_logistic_sigmoid():
     # Check correctness and robustness of logistic sigmoid implementation
     def naive_log_logistic(x):
         return np.log(expit(x))
 
     x = np.linspace(-2, 2, 50)
-    assert_array_almost_equal(log_logistic(x), naive_log_logistic(x))
+    warn_msg = "`log_logistic` is deprecated and will be removed"
+    with pytest.warns(FutureWarning, match=warn_msg):
+        assert_array_almost_equal(log_logistic(x), naive_log_logistic(x))
 
     extreme_x = np.array([-100.0, 100.0])
-    assert_array_almost_equal(log_logistic(extreme_x), [-100, 0])
+    with pytest.warns(FutureWarning, match=warn_msg):
+        assert_array_almost_equal(log_logistic(extreme_x), [-100, 0])
 
 
 @pytest.fixture()
@@ -970,33 +981,38 @@ def test_stable_cumsum():
 
 
 @pytest.mark.parametrize(
-    "A_array_constr", [np.array, sparse.csr_matrix], ids=["dense", "sparse"]
+    "A_container",
+    [np.array, *CSR_CONTAINERS],
+    ids=["dense"] + [container.__name__ for container in CSR_CONTAINERS],
 )
 @pytest.mark.parametrize(
-    "B_array_constr", [np.array, sparse.csr_matrix], ids=["dense", "sparse"]
+    "B_container",
+    [np.array, *CSR_CONTAINERS],
+    ids=["dense"] + [container.__name__ for container in CSR_CONTAINERS],
 )
-def test_safe_sparse_dot_2d(A_array_constr, B_array_constr):
+def test_safe_sparse_dot_2d(A_container, B_container):
     rng = np.random.RandomState(0)
 
     A = rng.random_sample((30, 10))
     B = rng.random_sample((10, 20))
     expected = np.dot(A, B)
 
-    A = A_array_constr(A)
-    B = B_array_constr(B)
+    A = A_container(A)
+    B = B_container(B)
     actual = safe_sparse_dot(A, B, dense_output=True)
 
     assert_allclose(actual, expected)
 
 
-def test_safe_sparse_dot_nd():
+@pytest.mark.parametrize("csr_container", CSR_CONTAINERS)
+def test_safe_sparse_dot_nd(csr_container):
     rng = np.random.RandomState(0)
 
     # dense ND / sparse
     A = rng.random_sample((2, 3, 4, 5, 6))
     B = rng.random_sample((6, 7))
     expected = np.dot(A, B)
-    B = sparse.csr_matrix(B)
+    B = csr_container(B)
     actual = safe_sparse_dot(A, B)
     assert_allclose(actual, expected)
 
@@ -1004,31 +1020,30 @@ def test_safe_sparse_dot_nd():
     A = rng.random_sample((2, 3))
     B = rng.random_sample((4, 5, 3, 6))
     expected = np.dot(A, B)
-    A = sparse.csr_matrix(A)
+    A = csr_container(A)
     actual = safe_sparse_dot(A, B)
     assert_allclose(actual, expected)
 
 
 @pytest.mark.parametrize(
-    "A_array_constr", [np.array, sparse.csr_matrix], ids=["dense", "sparse"]
+    "container",
+    [np.array, *CSR_CONTAINERS],
+    ids=["dense"] + [container.__name__ for container in CSR_CONTAINERS],
 )
-def test_safe_sparse_dot_2d_1d(A_array_constr):
+def test_safe_sparse_dot_2d_1d(container):
     rng = np.random.RandomState(0)
-
     B = rng.random_sample((10))
 
     # 2D @ 1D
     A = rng.random_sample((30, 10))
     expected = np.dot(A, B)
-    A = A_array_constr(A)
-    actual = safe_sparse_dot(A, B)
+    actual = safe_sparse_dot(container(A), B)
     assert_allclose(actual, expected)
 
     # 1D @ 2D
     A = rng.random_sample((10, 30))
     expected = np.dot(B, A)
-    A = A_array_constr(A)
-    actual = safe_sparse_dot(B, A)
+    actual = safe_sparse_dot(B, container(A))
     assert_allclose(actual, expected)
 
 
